#
# This file is part of python sample package for the BROADWAY2 package.
#
# MIT License

# Copyright (c) 2013-2019 FibreCode GmbH <info@fibrecode.com>

# Permission is hereby granted, free of charge, to any person obtaining a copy
# of this software and associated documentation files (the "Software"), to deal
# in the Software without restriction, including without limitation the rights
# to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
# copies of the Software, and to permit persons to whom the Software is
# furnished to do so, subject to the following conditions:

# The above copyright notice and this permission notice shall be included in all
# copies or substantial portions of the Software.

# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
# SOFTWARE.


import sys
import time
import binascii
import struct

from broadway2 import Broadway, SmiAccess


def main(args):
    
    base = Broadway()

    aList = base.get_tool_adapters()

    print('Num Tool Adapters found: {0}'.format(len(aList)))
    print()

    # verify to have just one Tool-Adapter connected
    for adapter in aList:
        inst, friendlyName = adapter

        dev = base.open_control(inst)

        smi = SmiAccess()

        smi.connect(dev)

        res = smi.detect()
        if len(res) > 0:
            print('Phys on Tool-Adapter: {0}'.format(friendlyName))
            print('-'*60)
            for phy in res:
                print('PhyID: {0} {1:04X} / {2:04X}'.format(phy[0], phy[1], phy[2]))
            print()

        
        # finaly close device
        base.close(dev)


if __name__ == "__main__":
    main(sys.argv)


